package db

import (
	"errors"
	"math"
	"strings"

	"3g3e.com/ymdx/glog"
	"3g3e.com/ymdx/lib"
	"3g3e.com/zmy/ck.platform.shop/common"
	"github.com/jmoiron/sqlx"
)

var sdb *sqlx.DB

func InitDB(dirverName, dataSource string) (*sqlx.DB, error) {
	if sdb != nil {
		sdb.Close()
		sdb = nil
	}

	if dirverName == "sqlite3" && dataSource != ":memory:" {
		dataSource = lib.EnsureFilePath(dataSource)
	}

	var err error
	sdb, err = sqlx.Open(dirverName, dataSource)
	if err != nil {
		glog.Infoln(err)
		return nil, err
	}
	sdb.SetMaxOpenConns(300)
	sdb.SetMaxIdleConns(30)
	sdb.Ping()

	//create table

	return sdb, nil
}
func PayOrderFld(sess map[string]interface{}, utype string, muid int, uid int, cpnum float64, orderId string, about string) error {

	if utype == "user" {

		sql := "SELECT user_money FROM user WHERE uid=? LIMIT 0, 1"
		rows, err := sdb.Queryx(sql, muid)
		if err != nil {
			glog.Errorln(err)
			return err
		}

		rs := common.Rows(rows)
		rows.Close()
		if len(rs) == 0 {
			return errors.New("1#len(rs) == 0")
		}
		mone := rs[0]

		if cpnum == 0 {
			about = "您已成功获得此次充值!"
		}
		if cpnum < 0 {
			needMoney := lib.ValueToFloat(mone["user_money"], 0.0) - math.Abs(cpnum/10000)
			wtime := lib.Now().Format("2006-01-02 15:04:05")
			sqli := "INSERT INTO paycouponlog (uid,order_sn,muid,user_money,cpcr,type_desc,cpabout,status,now_count,wtime) VALUES (?,?,?,?,?,?,?,?,?,?)"
			_, err := sdb.Exec(sqli,
				uid,
				orderId,
				muid,
				cpnum/10000,
				"",
				"获得充值",
				about,
				1,
				needMoney,
				wtime)
			if err != nil {
				glog.Errorln(err)
				return err
			}

			DecUserMoney(muid, math.Abs(cpnum), about, needMoney, sess)
		} else {
			needMoney := lib.ValueToFloat(mone["user_money"], 0.0) + math.Abs(cpnum/10000)
			wtime := lib.Now().Format("2006-01-02 15:04:05")
			sqli := "INSERT INTO paycouponlog (uid,order_sn,muid,user_money,cpcr,type_desc,cpabout,status,now_count,wtime) VALUES (?,?,?,?,?,?,?,?,?,?)"
			_, err := sdb.Exec(sqli,
				uid,
				orderId,
				muid,
				cpnum/10000,
				"",
				"获得充值",
				about,
				1,
				needMoney,
				wtime)
			if err != nil {
				glog.Errorln(err)
				return err
			}

			AddUserMoney(muid, cpnum, about, needMoney, sess)
		}

	} else if utype == "euser" {
		sql := "SELECT user_money FROM euser WHERE uid=? LIMIT 0, 1"
		rows, err := sdb.Queryx(sql, muid)
		if err != nil {
			glog.Errorln(err)
			return err
		}
		rs := common.Rows(rows)
		rows.Close()

		if len(rs) == 0 {
			return errors.New("2#len(rs) == 0")
		}
		mone := rs[0]

		if cpnum == 0 {
			about = "您已成功获得此次充值!"
		}
		if cpnum < 0 {
			needMoney := lib.ValueToFloat(mone["user_money"], 0.0) - math.Abs(cpnum/10000)
			wtime := lib.Now().Format("2006-01-02 15:04:05")
			sqli := "INSERT INTO epaycouponlog (uid,order_sn,muid,user_money,cpcr,type_desc,cpabout,status,now_count,wtime) VALUES (?,?,?,?,?,?,?,?,?,?)"
			_, err := sdb.Exec(sqli,
				uid,
				orderId,
				muid,
				cpnum/10000,
				"",
				"获得充值",
				about,
				1,
				needMoney,
				wtime,
			)
			if err != nil {
				glog.Errorln(err)
				return err
			}
			DecEuserMoney(muid, math.Abs(cpnum), about, needMoney, sess)
		} else {
			needMoney := lib.ValueToFloat(mone["user_money"], 0.0) + math.Abs(cpnum/10000)
			wtime := lib.Now().Format("2006-01-02 15:04:05")
			sqli := "INSERT INTO epaycouponlog (uid,order_sn,muid,user_money,cpcr,type_desc,cpabout,status,now_count,wtime) VALUES (?,?,?,?,?,?,?,?,?,?)"
			_, err := sdb.Exec(sqli,
				uid,
				orderId,
				muid,
				cpnum/10000,
				"",
				"获得充值",
				about,
				1,
				needMoney,
				wtime,
			)
			if err != nil {
				glog.Errorln(err)
				return err
			}
			AddEuserMoney(muid, cpnum, about, needMoney, sess)
		}
	}

	return nil
}
func AddUserMoney(uid int, num float64, title string, now_count float64, sess map[string]interface{}) bool {
	re := false
	if !lib.ValueIsEmpty(uid) && num >= 0 {
		re = AddPayLog(
			uid,
			num,
			title,
			now_count,
			sess,
		)
	}
	return re
}

func DecUserMoney(uid int, num float64, title string, now_count float64, sess map[string]interface{}) bool {
	re := false
	if !lib.ValueIsEmpty(uid) && num >= 0 {
		re = AddPayLog(
			uid,
			(-1)*num,
			title,
			now_count,
			sess,
		)
	}
	return re
}
func AddPayLog(userId int, userMoney float64, changeDesc string, now_count float64, sess map[string]interface{}) bool {

	if userMoney < 0 {
		gcount := GetCount("SELECT COUNT(*) FROM user Where uid = ? and user_money >= ?", userId, math.Abs(userMoney/10000))
		if gcount == 0 {
			return false
		}
	}

	/* 插入帐户变动记录 */
	changeTime := lib.Now().Format("2006-01-02 15:04:05")
	sql := "INSERT INTO payaccountlog (user_id,user_money,change_desc,change_time,change_type,now_count) VALUES (?,?,?,?,?,?)"
	_, err := sdb.Exec(sql,
		userId,
		userMoney/10000,
		changeDesc,
		changeTime,
		99,
		now_count,
	)
	if err != nil {
		glog.Errorln(err)
		return false
	}
	/* 更新用户信息 */

	sqli := "UPDATE user SET user_money=user_money+? WHERE uid=?"
	_, err = sdb.Exec(sqli,
		userMoney/10000,
		userId,
	)
	if err != nil {
		glog.Errorln(err)
		return false
	}
	if sess != nil {
		sqls := "SELECT user_money FROM user WHERE uid=? LIMIT 0, 1"
		rows, err := sdb.Queryx(sqls, userId)
		if err != nil {
			glog.Errorln(err)
			return false
		}
		rs := common.Rows(rows)
		rows.Close()

		if len(rs) == 0 {
			glog.Errorln("#重大错误")
			return false
		}
		mone := rs[0]
		sess["user_money"] = mone["user_money"]
	}
	return true
}

func AddEPayLog(userId int, userMoney float64, changeDesc string, now_count float64, sess map[string]interface{}) bool {

	if userMoney < 0 && userId != 1 {
		gcount := GetCount("SELECT COUNT(*) FROM euser Where uid = ? and user_money >= ?", userId, math.Abs(userMoney/10000))
		if gcount == 0 {
			return false
		}
	}

	/* 插入帐户变动记录 */
	changeTime := lib.Now().Format("2006-01-02 15:04:05")
	sql := "INSERT INTO epayaccountlog (user_id,user_money,change_desc,change_time,change_type,now_count) VALUES (?,?,?,?,?,?)"
	_, err := sdb.Exec(sql,
		userId,
		userMoney/10000,
		changeDesc,
		changeTime,
		99,
		now_count,
	)
	if err != nil {
		glog.Errorln(err)
		return false
	}
	/* 更新用户信息 */

	sqli := "UPDATE euser SET user_money=user_money+?,sys_money=sys_money+? WHERE uid=?"
	_, err = sdb.Exec(sqli,
		userMoney/10000,
		userMoney/10000,
		userId,
	)
	if err != nil {
		glog.Errorln(err)
		return false
	}
	if sess != nil {
		sqls := "SELECT user_money FROM euser WHERE uid=? LIMIT 0, 1"
		rows, err := sdb.Queryx(sqls, userId)
		if err != nil {
			glog.Errorln(err)
			return false
		}
		rs := common.Rows(rows)
		rows.Close()

		if len(rs) == 0 {
			glog.Errorln("#重大错误")
			return false
		}
		mone := rs[0]
		sess["user_money"] = mone["user_money"]
		sess["sys_money"] = mone["sys_money"]
	}
	return true
}
func GetCount(sql string, args ...interface{}) int {
	strings.Replace(sql, "count(*)", "COUNT(*)", -1)
	rows, err := sdb.Queryx(sql, args...)
	if err != nil {
		return 0
	}
	rs := common.Rows(rows)
	rows.Close()

	if len(rs) == 0 {
		return 0
	}

	return lib.ValueToInt(rs[0]["COUNT(*)"], 0)
}
func DecEuserMoney(uid int, num float64, title string, now_count float64, sess map[string]interface{}) bool {

	re := false
	if !lib.ValueIsEmpty(uid) && num >= 0 {
		re = AddEPayLog(
			uid,
			(-1)*num,
			title,
			now_count,
			sess,
		)
	}
	return re
}
func AddEuserMoney(uid int, num float64, title string, now_count float64, sess map[string]interface{}) bool {

	re := false
	if !lib.ValueIsEmpty(uid) && num >= 0 {
		re = AddEPayLog(
			uid,
			num,
			title,
			now_count,
			sess,
		)
	}
	return re
}

func GETEuser(eid int64) (map[string]interface{}, error) {
	sql := "SELECT user_money FROM euser WHERE uid=? LIMIT 0, 1"
	rows, err := sdb.Queryx(sql, eid)
	if err != nil {
		glog.Errorln(err)
		return nil, err
	}
	rs := common.Rows(rows)
	rows.Close()

	if len(rs) == 0 {
		return nil, errors.New("2#len(rs) == 0")
	}
	sess := rs[0]
	return sess, nil
}
