package keys

import (
	"path/filepath"
	"sync"

	baapi "3g3e.com/rechy/ck.ba.api"
	"3g3e.com/rechy/visdb"
	"3g3e.com/ymdx/lib"
)

func TokenSaverInitBySyncMap() {
	baapi.ModTokenSaver = &sync.Map{}
}

func TokenSaverInit(baseDir string) {
	baapi.ModTokenSaver = newVisdbTokenSaver(baseDir)
}

type visdbTokenSaver struct {
	kv      *visdb.KV
	baseDir string
}

func newVisdbTokenSaver(baseDir string) *visdbTokenSaver {
	return &visdbTokenSaver{
		kv:      visdb.NewKV(filepath.Join(baseDir, "db_pub_token_saver"), 3),
		baseDir: baseDir,
	}
}

func (ts *visdbTokenSaver) Store(k interface{}, v interface{}) {
	ts.kv.Put(lib.ValueToString(k, ""), v)
}

func (ts *visdbTokenSaver) Load(k interface{}) (interface{}, bool) {
	b, err := ts.kv.Get(lib.ValueToString(k, ""))
	if err != nil {
		return nil, false
	}
	return b, true
}

func (ts *visdbTokenSaver) Delete(k interface{}) {
	ts.kv.Delete(lib.ValueToString(k, ""))
}

func Store(k interface{}, v interface{}) {
	baapi.ModTokenSaver.Store(k, v)
}

func Load(k interface{}) (interface{}, bool) {
	return baapi.ModTokenSaver.Load(k)
}
