package models

import (
	"context"
	"fmt"
	"time"

	cfg "3g3e.com/ymdx/config"
	"3g3e.com/ymdx/glog"
	sdb "3g3e.com/zmy/ck.platform.shop/db"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
)

var Db *gorm.DB
var tables []interface{}

type dbLogger struct{}

func (w *dbLogger) LogMode(logger.LogLevel) logger.Interface {
	return w
}

func (w *dbLogger) Info(ctx context.Context, format string, args ...interface{}) {
	glog.InfoExtDepthln("db", -4, fmt.Sprintf(format, args...))
}

func (w *dbLogger) Warn(ctx context.Context, format string, args ...interface{}) {
	glog.WarningExtDepthln("db", -4, fmt.Sprintf(format, args...))
}

func (w *dbLogger) Error(ctx context.Context, format string, args ...interface{}) {
	glog.ErrorExtDepthln("db", -4, fmt.Sprintf(format, args...))
}

func (w *dbLogger) Trace(ctx context.Context, begin time.Time, fc func() (sql string, rowsAffected int64), err error) {
	sql, affw := fc()
	glog.InfoExtDepthln("db", -4, sql, affw, err)
}

func Init() error {
	driver := cfg.String("db.driver", "mysql")
	glog.InfoExtln("db", driver)

	source := cfg.String("db.source", "ckdx01:WAdijEspwEE@tcp(221.180.226.54:3389)/ck_dx?charset=utf8&parseTime=True&loc=Local")
	db, err := gorm.Open(mysql.New(mysql.Config{
		DSN: source,
	}), &gorm.Config{
		Logger: &dbLogger{},
	})
	if err != nil {
		return err
	}
	Db = db
	sdb.InitDB(driver, source)

	Db.AutoMigrate(tables...)
	//Db.DB().SetMaxIdleConns(10)
	//Db.DB().SetMaxOpenConns(100)
	return nil
}

func CloseDB() {

}

func SqlInStrFromArrayString(items []string) string {
	ll := len(items)
	str := ""
	for i := 0; i < ll; i++ {
		idStr := items[i]
		if idStr == "" {
			continue
		}
		if i > 0 {
			str = str + "','" + idStr
		} else {
			str = "('" + idStr
		}
	}
	if len(str) > 0 {
		str = str + "')"
	} else {
		str = "(-1)"
	}
	return str
}
