package models

import (
	"time"

	"3g3e.com/ymdx/lib"
)

// 标签类别表
type ShopCategory struct {
	Id         int64     `gorm:"primary_key" json:"id"`
	Eid        int64     `gorm:"type:bigint(20)" json:"eid"`    //商户EID
	Name       string    `gorm:"type:char (64)" json:"name"`    //类别名称
	Pic        string    `gorm:"type:varchar (250)" json:"pic"` //类别图标
	Pid        int64     `gorm:"type:bigint(20)" json:"pid"`    //父ID
	CreateTime time.Time `gorm:"type:datetime" json:"create_time"`
	UpdateTime time.Time `gorm:"type:datetime" json:"update_time"`
}

func init() {
	tables = append(tables, &ShopCategory{})
}

func (ShopCategory) TableName() string {
	return "shop_category"
}

func InsertShopCategory(data map[string]interface{}) (int64, error) {
	result := Db.Create(&ShopCategory{
		Name:       lib.ValueToString(data["name"], ""),
		Pid:        lib.ValueToInt64(data["pid"], 0),
		CreateTime: time.Now(),
	})

	return result.Statement.Dest.(*ShopCategory).Id, result.Error
}

func ShopCategoryOne(wh string, args ...interface{}) (results ShopCategory) {
	Db.Table("shop_category").Where(wh, args...).First(&results)
	return
}

func ShopCategoryList(wh string, args ...interface{}) []ShopCategory {
	var res []ShopCategory
	Db.Table("shop_category").Where(wh, args...).Order("id ASC").Find(&res)
	return res
}

func ShopCategoryListPg(wh string, pagesize int, pg int, args ...interface{}) (int, []ShopCategory, error) {
	var list []ShopCategory
	Db.Table("shop_category").Where(wh, args...).Limit(pagesize).Offset(pagesize * (pg - 1)).Find(&list)
	var count int64
	Db.Table("shop_category").Where(wh, args...).Count(&count)
	return int(count), list, nil
}

func UpdateShopCategory(wh string, data map[string]interface{}, args ...interface{}) error {
	result := Db.Table("shop_category").Where(wh, args...).Updates(data)
	return result.Error
}

func DeleteShopCategory(wh string, args ...interface{}) error {
	res := Db.Table("shop_category").Where(wh, args...).Delete(&ShopCategory{})
	return res.Error
}
