package models

import (
	"time"

	"3g3e.com/ymdx/lib"
)

// 标签类别表
type ShopProduct struct {
	Id           int64     `gorm:"primary_key" json:"id"`
	Eid          int64     `gorm:"type:bigint(20)" json:"eid"`                //商户EID
	Cid          int64     `gorm:"type:bigint(20)" json:"cid"`                //类别ID
	Name         string    `gorm:"type:varchar (250)" json:"name"`            //产品名称
	Price        float64   `gorm:"type:double(16,2)" json:"price"`            //现价
	Cost         float64   `gorm:"type:double(16,2)" json:"cost"`             //原价
	Postage      float64   `gorm:"type:double(16,2)" json:"postage"`          //邮费
	ImageUrl     string    `gorm:"type:text" json:"image_url"`                //展示图片
	ImageDetail  string    `gorm:"type:text" json:"image_detail"`             //详情图片
	Remark       string    `gorm:"type:varchar (250)" json:"remark"`          //备注
	KeyWord      string    `gorm:"type:varchar (250)" json:"key_word"`        //关键词
	SkuCode      string    `gorm:"type:varchar (250)" json:"sku_code"`        //商品编码
	Stock        int64     `gorm:"type:bigint(20)" json:"stock"`              //库存
	Sales        int64     `gorm:"type:bigint(20)" json:"sales"`              //销量
	Sort         int64     `gorm:"type:int(20)" json:"sort"`                  //排序
	Status       int64     `gorm:"type:int(1); default:'0'" json:"status"`    //状态 1上架 0下架
	IsDelete     int64     `gorm:"type:int(1); default:'0'" json:"is_delete"` //状态 1删除 0正常
	GoodsService string    `gorm:"type:text" json:"goods_service"`            //服务承诺
	CreateTime   time.Time `gorm:"type:datetime" json:"create_time"`
	UpdateTime   time.Time `gorm:"type:datetime" json:"update_time"`
}

func init() {
	tables = append(tables, &ShopProduct{})
}

func (ShopProduct) TableName() string {
	return "shop_product"
}

func InsertShopProduct(data map[string]interface{}) (int64, error) {
	result := Db.Create(&ShopProduct{
		Eid:          lib.ValueToInt64(data["eid"], 0),
		Cid:          lib.ValueToInt64(data["cid"], 0),
		Name:         lib.ValueToString(data["name"], ""),
		Price:        lib.ValueToFloat(data["price"], 0),
		Cost:         lib.ValueToFloat(data["cost"], 0),
		Postage:      lib.ValueToFloat(data["postage"], 0),
		ImageUrl:     lib.ValueToString(data["image_url"], ""),
		ImageDetail:  lib.ValueToString(data["image_detail"], ""),
		Remark:       lib.ValueToString(data["remark"], ""),
		KeyWord:      lib.ValueToString(data["key_word"], ""),
		SkuCode:      lib.ValueToString(data["sku_code"], ""),
		Stock:        lib.ValueToInt64(data["stock"], 0),
		Sales:        lib.ValueToInt64(data["sales"], 0),
		Sort:         lib.ValueToInt64(data["sort"], 0),
		Status:       lib.ValueToInt64(data["status"], 0),
		GoodsService: lib.ValueToString(data["goods_service"], ""),
		CreateTime:   time.Now(),
	})

	return result.Statement.Dest.(*ShopProduct).Id, result.Error
}
