package tpl

import (
	"bytes"
	"errors"
	"regexp"
	"strings"

	"3g3e.com/ymdx/lib"
)

func TemplateToContent(template, paramsJsonStr string, contents ...string) (string, error) {
	template = strings.TrimSpace(template)
	if template == "" {
		return "", errors.New("empty template")
	}

	if template == "*" {
		if len(contents) < 1 || contents[0] == "" {
			return "", errors.New("no sms content")
		}
		return strings.TrimSpace(contents[0]), nil
	}

	params := lib.JsonDecode([]byte(paramsJsonStr))
	r := regexp.MustCompile(`\$\{([a-zA-Z0-9_]+)\}`)
	tpl := []byte(template)
	arr := r.FindAllSubmatchIndex(tpl, -1)

	pre := 0
	buf := bytes.NewBuffer(nil)
	for _, v := range arr {
		if len(v) < 4 {
			continue
		}

		buf.Write(tpl[pre:v[0]])
		key := template[v[2]:v[3]]
		val, ok := params[string(key)]
		if !ok {
			return "", errors.New("lost var")
		}
		pre = v[1]
		valStr := lib.ValueToString(val, "")
		if valStr == "" {
			return "", errors.New("unknown sms var value")
		}
		buf.WriteString(valStr)
	}
	buf.Write(tpl[pre:])
	return buf.String(), nil
}

// 模板变量返回
func TemplateVars(template string) ([]string, error) {
	r := regexp.MustCompile(`\$\{([a-zA-Z0-9_]+)\}`)
	arr := r.FindAllStringSubmatch(template, -1)
	sz := len(arr)
	if sz < 1 {
		return nil, nil
	}

	mp := map[string]string{}
	re := make([]string, 0)
	for _, v := range arr {
		if len(v) < 2 {
			continue
		}
		_, ok := mp[v[1]]
		if ok {
			return nil, errors.New("vars count error")
		}
		mp[v[1]] = v[0]
		re = append(re, v[1])
	}

	return re, nil
}

// 模板书写是否有效
func TemplateValid(template string) bool {
	r := regexp.MustCompile(`\$\{([a-zA-Z0-9_]+)\}`)
	arr := r.FindAllStringSubmatch(template, -1)
	mp := map[string]string{}
	for _, v := range arr {
		if len(v) < 2 {
			continue
		}
		mp[v[1]] = v[0]
	}

	sz := len(mp)
	if sz != len(arr) {
		return false
	}

	r2 := regexp.MustCompile(`\$\{`)
	arr2 := r2.FindAllString(template, -1)
	return sz == len(arr2)
}
