package utils

import (
	"fmt"
	"math/rand"
	"reflect"
	"regexp"
	"strconv"
	"strings"
	"time"

	"github.com/dgrijalva/jwt-go"
	guuid "github.com/satori/go.uuid"
	"golang.org/x/crypto/bcrypt"
)

func ValueIsEmpty(str string) bool {

	if str == "" || len(str) == 0 {
		return true
	}
	return false
}
func ValueToInt(in interface{}, def int) int {
	if in == nil {
		return def
	}
	switch in.(type) {
	case int:
		return in.(int)
	case int8:
		return int(in.(int8))
	case int16:
		return int(in.(int16))
	case int32:
		return int(in.(int32))
	case int64:
		return int(in.(int64))
	case uint:
		return int(in.(uint))
	case uint8:
		return int(in.(uint8))
	case uint16:
		return int(in.(uint16))
	case uint32:
		return int(in.(uint32))
	case uint64:
		return int(in.(uint64))
	case float32:
		return int(in.(float32))
	case float64:
		return int(in.(float64))
	case []byte:
		ii, _ := strconv.ParseFloat(string(in.([]byte)), 64)
		return int(ii)
	case string:
		ii, _ := strconv.ParseFloat(in.(string), 64)
		return int(ii)
	case *[]byte:
		ii, _ := strconv.ParseFloat(string(*(in.(*[]byte))), 64)
		return int(ii)
	case *string:
		ii, _ := strconv.ParseFloat(*(in.(*string)), 64)
		return int(ii)
	default:
	}
	return def
}
func ValueToString(in interface{}, def string) string {
	return ValueToStringIn(in, def, 0)
}

func ValueToStringIn(in interface{}, def string, fixed int) string {
	if in == nil {
		return def
	}
	switch in.(type) {
	case int:
		return strconv.FormatInt(int64(in.(int)), 10)
	case int8:
		return strconv.FormatInt(int64(in.(int8)), 10)
	case int16:
		return strconv.FormatInt(int64(in.(int16)), 10)
	case int32:
		return strconv.FormatInt(int64(in.(int32)), 10)
	case int64:
		return strconv.FormatInt(in.(int64), 10)
	case uint:
		return strconv.FormatUint(uint64(in.(uint)), 10)
	case uint8:
		return strconv.FormatUint(uint64(in.(uint8)), 10)
	case uint16:
		return strconv.FormatUint(uint64(in.(uint16)), 10)
	case uint32:
		return strconv.FormatUint(uint64(in.(uint32)), 10)
	case uint64:
		return strconv.FormatUint(in.(uint64), 10)
	case float32:
		if fixed > 0 {
			return strconv.FormatFloat(float64(in.(float32)), 'f', fixed, 32)
		}
		return strconv.FormatFloat(float64(in.(float32)), 'f', 10, 32)
	case float64:
		if fixed > 0 {
			return strconv.FormatFloat(in.(float64), 'f', fixed, 64)
		}
		return strconv.FormatFloat(in.(float64), 'f', 10, 64)
	case []byte:
		return string(in.([]byte))
	case string:
		return in.(string)
	case *[]byte:
		return string(*(in.(*[]byte)))
	case *string:
		return *(in.(*string))
	default:
	}
	return def
}

// 判断某一个值是否在切片中
func In_array(val interface{}, array interface{}) (exists bool, index int) {
	exists = false
	index = -1
	switch reflect.TypeOf(array).Kind() {
	case reflect.Slice:
		s := reflect.ValueOf(array)
		for i := 0; i < s.Len(); i++ {
			if reflect.DeepEqual(val, s.Index(i).Interface()) == true {
				index = i
				exists = true
				return
			}
		}
	}
	return
}

// 数组转换成string类型
func ArrayToString(wh []string) string {
	whereStr := ""
	arrLen := len(wh)
	for i := 0; i < arrLen; i++ {
		whereStr = whereStr + " " + wh[i] + " AND"
	}
	whereStr = strings.TrimRight(whereStr, " AND")
	return whereStr
}

// 生成随机方案
func GetOrderNum() string {

	// 获取纳秒数
	usec := time.Now().UnixNano()
	usecStr := strconv.FormatInt(usec, 10)[13:16]

	// 当天凌晨时间
	str := time.Now().Format("2006-01-02")
	ti, _ := time.Parse("2006-01-02", str)

	// 当前时间和凌晨时间差的秒数
	timeNum := time.Now().Unix() - ti.Unix()
	sec := fmt.Sprintf("%05d", timeNum)

	// 生成随机数并自动填充
	sn := fmt.Sprintf("%05d", rand.Intn(100000))

	num := time.Now().Format("060102") + sec + usecStr + sn
	return num
}

// 数据库中的时间转换
func StrTimeTostr(str string) string {
	time := strings.Replace(str, "T", " ", -1)
	time = time[0:19]
	return time
}

/**
 * 解析 token
 */
func ParseToken(tokenSrt string, SecretKey []byte) (id string, name string, err error) {
	var token *jwt.Token
	token, err = jwt.Parse(tokenSrt, func(*jwt.Token) (interface{}, error) {
		return SecretKey, nil
	})
	if !token.Valid { //服务端验证token是否有效
		if ve, ok := err.(*jwt.ValidationError); ok {
			if ve.Errors&jwt.ValidationErrorMalformed != 0 {
				fmt.Println("错误的token")
				return
			} else if ve.Errors&(jwt.ValidationErrorExpired|jwt.ValidationErrorNotValidYet) != 0 {
				fmt.Println("token过期或未启用")
				return
			} else {
				fmt.Println("无法处理这个token", err)
				return
			}
		}
	}
	// token信息
	claimsa, _ := token.Claims.(jwt.MapClaims)
	id = claimsa["id"].(string)
	name = claimsa["name"].(string)
	return
}

// 加密
// 生成随机ordersn方案
func GetOrderSn() string {

	// 获取纳秒数
	usec := time.Now().UnixNano()
	usecStr := strconv.FormatInt(usec, 10)[13:16]

	// 当天凌晨时间
	str := time.Now().Format("2006-01-02")
	ti, _ := time.Parse("2006-01-02", str)

	// 当前时间和凌晨时间差的秒数
	timeNum := time.Now().Unix() - ti.Unix()
	sec := fmt.Sprintf("%05d", timeNum)

	// 生成随机数并自动填充
	sn := fmt.Sprintf("%05d", rand.Intn(100000))

	num := time.Now().Format("060102") + sec + usecStr + sn
	return num
}

// 加密密码
func GenUserHashPassword(password string) (string, error) {
	hash, err := bcrypt.GenerateFromPassword([]byte(password), bcrypt.DefaultCost)
	if err != nil {
		return "", err
	}
	return string(hash), nil
}

// 验证密码
func ComparePasswords(hashedPwd string, plainPwd string) error {
	byteHash := []byte(hashedPwd)
	bytePlain := []byte(plainPwd)
	err := bcrypt.CompareHashAndPassword(byteHash, bytePlain)
	return err

}

// 将rune数组用字符串常量替换
const letterBytes = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"

// 生成随机数
func RandStringBytes(n int) string {
	// GenRsaKey(1024)
	b := make([]byte, n)
	for i := range b {
		b[i] = letterBytes[rand.Intn(len(letterBytes))]
	}
	return string(b)
}

func GetOrderSnn() string {
	str := guuid.NewV4().String()
	return strings.ReplaceAll(str, "-", "")
}

func SplitString(s string, myStrings []rune) []string {
	Split := func(r rune) bool {
		for _, v := range myStrings {
			if v == r {
				return true
			}
		}
		return false
	}
	a := strings.FieldsFunc(s, Split)
	return a
}

// 提取模板内容
func ExtractContent(input string) []string {
	// 定义正则表达式模式
	pattern := `\${(.*?)\}` // 非贪婪模式，只匹配最小数量的字符
	// pattern := `/ $ (\w) }/gi`
	// 编译正则表达式
	re := regexp.MustCompile(pattern)

	// 查找所有匹配项并返回结果切片
	matches := re.FindAllStringSubmatch(input, -1)

	var result []string
	for _, match := range matches {
		re := regexp.MustCompile(`\$|\{|\}`) // 创建正则表达式对象，匹配大括号
		result = append(result, re.ReplaceAllString(match[0], ""))
	}

	return result
}

// 生成随机数
func GenValidateCodeNew(width int) string {
	numeric := [64]string{"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"}
	r := len(numeric)
	rand.Seed(time.Now().UnixNano())

	sb := ""
	for i := 0; i < width; i++ {
		sb += numeric[rand.Intn(r)]
	}
	return sb
}

func CheckMobile(phone string) (bool, string) {
	// todo
	valid := regexp.MustCompile("[0-9]")
	num := valid.FindAllString(phone, -1)
	phone = strings.Join(num, "")
	// 匹配规则
	// ^1第一位为一
	// [345789]{1} 后接一位345789 的数字
	// \\d \d的转义 表示数字 {9} 接9位
	// $ 结束符
	regRuler := "^1[3456789]{1}\\d{9}$"

	// 正则调用规则
	reg := regexp.MustCompile(regRuler)

	// 返回 MatchString 是否匹配
	return reg.MatchString(phone), phone

}

// 计算时间差（天数）
func SubDays(t1, t2 time.Time) (day int) {
	day = int(t1.Sub(t2).Hours() / 24)
	return
}
