package routers

import (
	"net/http"

	"3g3e.com/ymdx/glog"
	"3g3e.com/zmy/ck.platform.shop/api"
	"3g3e.com/zmy/ck.platform.shop/api/pub"

	"strings"

	// "3g3e.com/zmy/ck.platform.shop/controllers"
	baapi "3g3e.com/rechy/ck.ba.api"
	"3g3e.com/ymdx/lib"
	"3g3e.com/zmy/ck.platform.shop/keys"
	"github.com/gin-gonic/gin"
)

const (
	ApiPathBase = "/"
)

// Cors is 跨域
func Cors() gin.HandlerFunc {
	return func(c *gin.Context) {

		if c.Request == nil || !strings.HasPrefix(c.Request.URL.Path, ApiPathBase) {
			c.Next()
			return
		}

		origin := c.Request.Header.Get("Origin") //请求头部
		wh := c.Writer.Header()
		if origin != "" {
			wh.Set("Access-Control-Allow-Origin", origin)
		} else {
			wh.Set("Access-Control-Allow-Origin", "*")
		}
		wh.Set("Access-Control-Allow-Methods", "POST, GET, OPTIONS, PUT, DELETE, UPDATE")
		wh.Set("Access-Control-Allow-Headers", "Authorization, Content-Length, X-CSRF-Token, Token,session,X_Requested_With,Accept, Origin, Host, Connection, Accept-Encoding, Accept-Language,DNT, X-CustomHeader, Keep-Alive, User-Agent, X-Requested-With, If-Modified-Since, Cache-Control, Content-Type, Pragma")
		wh.Set("Access-Control-Expose-Headers", "Content-Length, Access-Control-Allow-Origin, Access-Control-Allow-Headers,Cache-Control,Content-Language,Content-Type,Expires,Last-Modified,Pragma,FooBar") // 跨域关键设置 让浏览器可以解析
		wh.Set("Access-Control-Max-Age", "172800")                                                                                                                                                           // 缓存请求信息 单位为秒
		wh.Set("Access-Control-Allow-Credentials", "true")                                                                                                                                                   // 设置返回格式是json

		//放行所有OPTIONS方法
		if c.Request.Method == "OPTIONS" {
			c.JSON(http.StatusOK, "Options Request!")
		}

		// 处理请求
		c.Next()
	}
}

func SetupRouter(router *gin.Engine) {
	// store := cookie.NewStore([]byte("secret"))
	// router.Use(Cors(), sessions.Sessions("mysession", store))
	/*******初始化操作权限路由范围 ******/
	baapi.InitBind(router, &PubHub{})
	// 初始化密钥管理器
	keys.GetKeyManager()

	/**********************/
	authMiddleware, err := getJwtAuth()
	if err != nil {
		glog.Fatalf("get jwt middleware err [%+v]", err)
		return
	}
	apiGroup := router.Group("/api")
	{
		// 无权限验证
		apiGroup.GET("/test", api.Test)
		apiGroup.POST("/login", authMiddleware.LoginHandler)
		apiGroup.POST("/upfile", pub.UpFile)
	}
	v1Master := router.Group(baapi.RouteApiMaster)
	v1Master.Use(authMiddleware.MiddlewareFunc())
	{
	}
	v1Enterprise := router.Group(baapi.RouteApiEnterprise)
	v1Enterprise.Use(authMiddleware.MiddlewareFunc())
	{
	}
	// 短信回调
	router.StaticFS("/img", newStaticDir(("/img"), lib.EnsureDir("./img"), ""))
	router.StaticFS("/fe", newStaticDir(("/fe"), lib.EnsureDir("./web/fe"), ""))
	router.StaticFS("/adm", newStaticDir(("/adm"), lib.EnsureDir("./wwwx"), "index.html"))
	router.GET("/", func(c *gin.Context) { c.Redirect(http.StatusFound, "/adm/") })
}
